//===========================================================================
//	CMidiInDeviceMenu Class Implementation
//
//	(C) Copyright, Paul A. Messick, 1996
//===========================================================================
#include "stdafx.h"
#include "MaxMidi.h"

//---------------------------------------------------------------------------
//	Constructors
//---------------------------------------------------------------------------
CMidiInDeviceMenu::CMidiInDeviceMenu(HMENU hWndMenu, UINT position, LPSTR mname, UINT baseMsg)
{
	Create(hWndMenu, position, mname, baseMsg);
}

//---------------------------------------------------------------------------
//	Create
//---------------------------------------------------------------------------
void CMidiInDeviceMenu::Create(HMENU hWndMenu, UINT position, LPSTR mname, UINT baseMsg)
{
	char name[MAXPNAMELEN];
	WORD wDevID;

	// create a new popup menu
	hPopupMenu = CreateMenu();
	idm_base = baseMsg;

	// and insert in the existing window menu
	InsertMenu(hWndMenu, position, MF_BYPOSITION | MF_POPUP | MF_ENABLED, (UINT)hPopupMenu, mname);

	// add all of the available input devices to the menu
	nMaxDevices = GetNumInDevices();
	for(wDevID = 0; wDevID < nMaxDevices; wDevID++)
	{
		GetDeviceName(wDevID, name);
		AppendMenu(hPopupMenu, MF_ENABLED | MF_STRING, baseMsg + wDevID, name);
	}
}

//---------------------------------------------------------------------------
//	GetDeviceName
//---------------------------------------------------------------------------
BOOL CMidiInDeviceMenu::GetDeviceName(WORD wDeviceID, LPSTR name)
{
	return GetMidiInDescription(wDeviceID, name);
}

//---------------------------------------------------------------------------
//	SelectDevice
//---------------------------------------------------------------------------
BOOL CMidiInDeviceMenu::SelectDevice(UINT id) 
{
	int i;
	CMenu m;
	
	// ignore this if no device is attached or if the id is not in range
	if(MidiIn == NULL || ((int)id > nMaxDevices && ((int)id < idm_base) || ((int)id > idm_base + nMaxDevices)))
		return FALSE;

	// get a temp CMenu to modify the menu state
	CMenu* Menu = m.FromHandle(hPopupMenu);

	// if the id is between 0 and the number of devices
	// then select the input based on the index, otherwise
	// it is a menu selection
	if((int)id <= nMaxDevices)
		id += idm_base;

	for(i = idm_base; i < (int)(idm_base + nMaxDevices); i++)
		if(i == (int)id)
		{
			//-----------------------------------------------------
			// close the currently open input device
			//-----------------------------------------------------
			MidiIn->Stop();
			MidiIn->Close();

			//-----------------------------------------------------
			// open midi in for use
			//-----------------------------------------------------
			MidiIn->Open(id - idm_base);
			MidiIn->Start();
			Menu->CheckMenuItem(i, MF_CHECKED);
		}
		else
			Menu->CheckMenuItem(i, MF_UNCHECKED);

	return TRUE;
}

