//-----------------------------------------------------------------------------
// Maximum MIDI Programmer's ToolKit - MxMidi32.DLL
// MaxMidi Flat Thunk Module
//
// Copyright (c) Paul A. Messick, 1994-1996
//
// Written by Paul A. Messick
//
// Thunks between 32-bit applications and the 16-bit MaxMidi DLL.  Also provides
// 32-bit SMF read and write functions.
//-----------------------------------------------------------------------------
#include <windows.h>
#include "MxDLL.h"

#ifdef WIN95_DLL
__declspec(dllexport) BOOL WINAPI MxMidi_ThunkConnect32(LPSTR pszDll16, LPSTR pszDll32, HINSTANCE hInst, DWORD dwReason);
#endif

//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------
PGetMaxMidiVersion32 GetMaxMidiVersion32;
PGetMidiOutDescription32 GetMidiOutDescription32;
POpenMidiOut32 OpenMidiOut32;
PResetMidiOut32 ResetMidiOut32;
PFlushMidiOut32 FlushMidiOut32;
PCloseMidiOut32 CloseMidiOut32;
PPutMidiOut32 PutMidiOut32;
PGetMidiInDescription32 GetMidiInDescription32;
POpenMidiIn32 OpenMidiIn32;
PStartMidiIn32 StartMidiIn32;
PStopMidiIn32 StopMidiIn32;
PCloseMidiIn32 CloseMidiIn32;
PGetMidiIn32 GetMidiIn32;
POpenSync32 OpenSync32;
PCloseSync32 CloseSync32;
PStopSync32 StopSync32;
PStartSync32 StartSync32;
PPauseSync32 PauseSync32;
PReStartSync32 ReStartSync32;
PSetTempo32 SetTempo32;
PSetResolution32 SetResolution32;
PGetTempo32 GetTempo32;
PGetResolution32 GetResolution32;
PGetPosition32 GetPosition32;

//*** V1.55 ***
PGetMidiInFilters32 GetMidiInFilters32;
PSetMidiInFilters32 SetMidiInFilters32;
PGetMidiOutFilters32 GetMidiOutFilters32;
PSetMidiOutFilters32 SetMidiOutFilters32;

// ***** CompuPhase addition
PSetBeatRatio32 SetBeatRatio32;

#ifndef WIN95_DLL
BOOL isNT = FALSE;

//============================================================================
//	Internal Functions
//============================================================================
//----------------------------------------------------------------------------
// IsWinNT
//
// Returns TRUE if running in any version of Windows NT, or FALSE if not.
// Since the ToolKit does not run in Win3.1, FALSE implies that the OS is
// Win95.
//----------------------------------------------------------------------------
BOOL IsWinNT(void)
{
	DWORD dwVersion;
	BOOL rc = FALSE;

	dwVersion = GetVersion();

	// if upper bit set, this is NT
	if(dwVersion < 0x80000000)
		rc = TRUE;

	// otherwise, it must be Win95
	return rc;
}
#endif

//----------------------------------------------------------------------------
//	DllMain
//
//	This is the 32-bit equivalent to "LibMain".  It connects with the 16-bit
//	DLL through the thunk layer.
//----------------------------------------------------------------------------
__declspec(dllexport) BOOL WINAPI DllMain(HINSTANCE hInst, DWORD dwReason, LPVOID lpReserved)
{
#ifndef WIN95_DLL
	HMODULE hDLLModule;
	WNDCLASS wndclass;		// Structure used to register class

	// determine if this is NT or Win95
	isNT = IsWinNT();
#endif

#ifdef WIN95_DLL // thunk to MxMidi16
	// call the entry point for the thunk layer
	if(!(MxMidi_ThunkConnect32("MXMIDI16.DLL", "MXMIDI95.DLL", hInst, dwReason)))
		return FALSE;
#endif

	switch(dwReason)
	{
		// DLL is attaching to the address space of the current process.
		case DLL_PROCESS_ATTACH:
#ifndef WIN95_DLL
			if(isNT)
			{
				// use local functions (and windows) for NT
				// setup window class parameters. Notice that this window is NEVER
				// displayed, so the display-specific parameters are set to NULL.
				wndclass.style	       = CS_GLOBALCLASS;
				wndclass.lpfnWndProc   = MidiInProc;
				wndclass.cbClsExtra    = 0;
				wndclass.cbWndExtra    = 0;
				wndclass.hInstance     = hInst;
				wndclass.hIcon	       = NULL;
				wndclass.hCursor       = NULL;
				wndclass.hbrBackground = NULL;
				wndclass.lpszMenuName  = NULL;
				wndclass.lpszClassName = MIDIINCLASS;
				RegisterClass(&wndclass);

				// setup hidden midi out window class parameters.  Notice that
				// this window is NEVER displayed, so the display-specific
				// parameters are set to NULL.
				wndclass.style	       = CS_GLOBALCLASS;
				wndclass.lpfnWndProc   = MidiOutProc;
				wndclass.cbClsExtra    = 0;
				wndclass.cbWndExtra    = 0;
				wndclass.hInstance     = hInst;
				wndclass.hIcon	       = NULL;
				wndclass.hCursor       = NULL;
				wndclass.hbrBackground = NULL;
				wndclass.lpszMenuName  = NULL;
				wndclass.lpszClassName = MIDIOUTCLASS;
				RegisterClass(&wndclass);

				hDLLModule = hInst;
			}
			else
			{
				hDLLModule = LoadLibrary("MxMidi95.dll");
				if(hDLLModule == NULL)
					return FALSE;
			}

			// for MxMidi32.dll get addresses for all of the local functions in MxMidi32.dll if NT, or from MxMidi95 if Win95
			GetMaxMidiVersion32	= (PGetMaxMidiVersion32)GetProcAddress(hDLLModule, "_GetMaxMidiVersionEx@0");
			GetMidiOutDescription32	= (PGetMidiOutDescription32)GetProcAddress(hDLLModule, "_GetMidiOutDescriptionEx@8");
			OpenMidiOut32	= (POpenMidiOut32)GetProcAddress(hDLLModule, "_OpenMidiOutEx@16");
			ResetMidiOut32	= (PResetMidiOut32)GetProcAddress(hDLLModule, "_ResetMidiOutEx@4");
			FlushMidiOut32	= (PFlushMidiOut32)GetProcAddress(hDLLModule, "_FlushMidiOutEx@4");
			CloseMidiOut32	= (PCloseMidiOut32)GetProcAddress(hDLLModule, "_CloseMidiOutEx@4");
			PutMidiOut32	= (PPutMidiOut32)GetProcAddress(hDLLModule, "_PutMidiOutEx@8");
			GetMidiInDescription32	= (PGetMidiInDescription32)GetProcAddress(hDLLModule, "_GetMidiInDescriptionEx@8");
			OpenMidiIn32	= (POpenMidiIn32)GetProcAddress(hDLLModule, "_OpenMidiInEx@16");
			StartMidiIn32	= (PStartMidiIn32)GetProcAddress(hDLLModule, "_StartMidiInEx@4");
			StopMidiIn32	= (PStopMidiIn32)GetProcAddress(hDLLModule, "_StopMidiInEx@4");
			CloseMidiIn32	= (PCloseMidiIn32)GetProcAddress(hDLLModule, "_CloseMidiInEx@4");
			GetMidiIn32	= (PGetMidiIn32)GetProcAddress(hDLLModule, "_GetMidiInEx@4");
			OpenSync32	= (POpenSync32)GetProcAddress(hDLLModule, "_OpenSyncEx@16");
			CloseSync32	= (PCloseSync32)GetProcAddress(hDLLModule, "_CloseSyncEx@4");
			StopSync32	= (PStopSync32)GetProcAddress(hDLLModule, "_StopSyncEx@4");
			StartSync32	= (PStartSync32)GetProcAddress(hDLLModule, "_StartSyncEx@4");
			PauseSync32	= (PPauseSync32)GetProcAddress(hDLLModule, "_PauseSyncEx@8");
			ReStartSync32	= (PReStartSync32)GetProcAddress(hDLLModule, "_ReStartSyncEx@4");
			SetTempo32	= (PSetTempo32)GetProcAddress(hDLLModule, "_SetTempoEx@8");
			SetResolution32	= (PSetResolution32)GetProcAddress(hDLLModule, "_SetResolutionEx@8");
			GetTempo32	= (PGetTempo32)GetProcAddress(hDLLModule, "_GetTempoEx@4");
			GetResolution32	= (PGetResolution32)GetProcAddress(hDLLModule, "_GetResolutionEx@4");
			GetPosition32	= (PGetPosition32)GetProcAddress(hDLLModule, "_GetPositionEx@8");

			// *** V1.55 ***
			GetMidiInFilters32 = (PGetMidiInFilters32)GetProcAddress(hDLLModule, "_GetMidiInFiltersEx@4");
			SetMidiInFilters32 = (PSetMidiInFilters32)GetProcAddress(hDLLModule, "_SetMidiInFiltersEx@8");
			GetMidiOutFilters32 = (PGetMidiOutFilters32)GetProcAddress(hDLLModule, "_GetMidiOutFiltersEx@4");
			SetMidiOutFilters32 = (PSetMidiOutFilters32)GetProcAddress(hDLLModule, "_SetMidiOutFiltersEx@8");

                        // ***** CompuPhase addition
			SetBeatRatio32	= (PSetBeatRatio32)GetProcAddress(hDLLModule, "_SetBeatRatioEx@12");

			// bail out if fcns can't be found
			if(GetMaxMidiVersion32 == NULL)
				return FALSE;
#else
			// for MxMidi95.dll get addresses for the thunks to the functions in MxMidi16.dll
			GetMaxMidiVersion32 = GetMaxMidiVersion16;
			GetMidiOutDescription32	= GetMidiOutDescription16;
			OpenMidiOut32 = OpenMidiOut16;
			ResetMidiOut32 = ResetMidiOut16;
			FlushMidiOut32 = FlushMidiOut16;
			CloseMidiOut32 = CloseMidiOut16;
			PutMidiOut32 = PutMidiOut16;
			GetMidiInDescription32	= GetMidiInDescription16;
			OpenMidiIn32 = OpenMidiIn16;
			StartMidiIn32 = StartMidiIn16;
			StopMidiIn32  = StopMidiIn16;
			CloseMidiIn32 = CloseMidiIn16;
			GetMidiIn32	= GetMidiIn16;
			OpenSync32	= OpenSync16;
			CloseSync32	= CloseSync16;
			StopSync32	= StopSync16;
			StartSync32	= StartSync16;
			PauseSync32	= PauseSync16;
			ReStartSync32 = ReStartSync16;
			SetTempo32 = SetTempo16;
			SetResolution32	= SetResolution16;
			GetTempo32	= GetTempo16;
			GetResolution32	= GetResolution16;
			GetPosition32 = GetPosition16;

			// *** V1.55 ***
			GetMidiInFilters32 = GetMidiInFilters16;
			SetMidiInFilters32 = SetMidiInFilters16;
			GetMidiOutFilters32 = GetMidiOutFilters16;
			SetMidiOutFilters32 = SetMidiOutFilters16;

                        // ***** CompuPhase addition
			SetBeatRatio32	= SetBeatRatio16;
#endif // WIN95_DLL
			break;

		// The calling process is detaching the DLL from its address space.
		case DLL_PROCESS_DETACH:
#ifndef WIN95_DLL
			if(isNT)
			{
				// only registered if NT
				UnregisterClass(MIDIINCLASS, hInst);
				UnregisterClass(MIDIOUTCLASS, hInst);
			}
#endif
			break;

		// A new thread is being created in the current process.
		case DLL_THREAD_ATTACH:
			break;

		// A thread is exiting cleanly.
		case DLL_THREAD_DETACH:
			break;
	}

	return TRUE;
}

//----------------------------------------------------------------------------
//	GetMaxMidiVersion
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) WORD WINAPI GetMaxMidiVersionEx(void)
#else
__declspec(dllexport) WORD WINAPI GetMaxMidiVersion(void)
#endif
{
	return GetMaxMidiVersion32();
}

//----------------------------------------------------------------------------
//	GetNumInDevices
//
//	For convenience: use in place of midiInGetNumDevs to avoid dealing with
//	MMSYSTEM directly.
//----------------------------------------------------------------------------
_declspec(dllexport) UINT WINAPI GetNumInDevices(void)
{
	return midiInGetNumDevs();
}

//----------------------------------------------------------------------------
//	GetNumOutDevices
//
//	For convenience: use in place of midiOutGetNumDevs to avoid dealing with
//	MMSYSTEM directly.
//----------------------------------------------------------------------------
_declspec(dllexport) UINT WINAPI GetNumOutDevices(void)
{
	return midiOutGetNumDevs();
}

//----------------------------------------------------------------------------
//	GetMidiOutDescription
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) BOOL WINAPI GetMidiOutDescriptionEx(WORD wDeviceID, LPSTR lpzDesc)
#else
__declspec(dllexport) BOOL WINAPI GetMidiOutDescription(WORD wDeviceID, LPSTR lpzDesc)
#endif
{
	return GetMidiOutDescription32(wDeviceID, lpzDesc);
}

//----------------------------------------------------------------------------
//	OpenMidiOut
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) HMOUT WINAPI OpenMidiOutEx(HWND hWnd, WORD wDeviceID, HSYNC hSync, DWORD dwFlags)
#else
__declspec(dllexport) HMOUT WINAPI OpenMidiOut(HWND hWnd, WORD wDeviceID, HSYNC hSync, DWORD dwFlags)
#endif
{
	return OpenMidiOut32(hWnd, wDeviceID, hSync, dwFlags);
}

//----------------------------------------------------------------------------
//	ResetMidiOut
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) WORD WINAPI ResetMidiOutEx(HMOUT hMidiOut)
#else
__declspec(dllexport) WORD WINAPI ResetMidiOut(HMOUT hMidiOut)
#endif
{
	return ResetMidiOut32(hMidiOut);
}

//----------------------------------------------------------------------------
//	FlushMidiout
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) void WINAPI FlushMidiOutEx(HMOUT hMidiOut)
#else
__declspec(dllexport) void WINAPI FlushMidiOut(HMOUT hMidiOut)
#endif
{
	FlushMidiOut32(hMidiOut);
}

//----------------------------------------------------------------------------
//	CloseMidiOut
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) WORD WINAPI CloseMidiOutEx(HMOUT hMidiOut)
#else
__declspec(dllexport) WORD WINAPI CloseMidiOut(HMOUT hMidiOut)
#endif
{
	return CloseMidiOut32(hMidiOut);
}

//----------------------------------------------------------------------------
//	PutMidiOut
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) WORD WINAPI PutMidiOutEx(HMOUT hMidiOut, LPMIDIEVENT lpMidiEvent)
#else
__declspec(dllexport) WORD WINAPI PutMidiOut(HMOUT hMidiOut, LPMIDIEVENT lpMidiEvent)
#endif
{
	return PutMidiOut32(hMidiOut, lpMidiEvent);
}

//----------------------------------------------------------------------------
// GetMidiInDescription
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) BOOL WINAPI GetMidiInDescriptionEx(WORD wDeviceID, LPSTR lpzDesc)
#else
__declspec(dllexport) BOOL WINAPI GetMidiInDescription(WORD wDeviceID, LPSTR lpzDesc)
#endif
{
	return GetMidiInDescription32(wDeviceID, lpzDesc);
}

//----------------------------------------------------------------------------
//	OpenMidiIn
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) HMIN WINAPI OpenMidiInEx(HWND hWnd, WORD wDeviceID, HSYNC hSync, DWORD dwFlags)
#else
__declspec(dllexport) HMIN WINAPI OpenMidiIn(HWND hWnd, WORD wDeviceID, HSYNC hSync, DWORD dwFlags)
#endif
{
	return OpenMidiIn32(hWnd, wDeviceID, hSync, dwFlags);
}

//----------------------------------------------------------------------------
//	StartMidiIn
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) WORD WINAPI StartMidiInEx(HMIN hMidiIn)
#else
__declspec(dllexport) WORD WINAPI StartMidiIn(HMIN hMidiIn)
#endif
{
	return StartMidiIn32(hMidiIn);
}

//----------------------------------------------------------------------------
//	StopMidiIn
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) WORD WINAPI StopMidiInEx(HMIN hMidiIn)
#else
__declspec(dllexport) WORD WINAPI StopMidiIn(HMIN hMidiIn)
#endif
{
	return StopMidiIn32(hMidiIn);
}

//----------------------------------------------------------------------------
//	CloseMidiIn
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) WORD WINAPI CloseMidiInEx(HMIN hMidiIn)
#else
__declspec(dllexport) WORD WINAPI CloseMidiIn(HMIN hMidiIn)
#endif
{
	return CloseMidiIn32(hMidiIn);
}

//----------------------------------------------------------------------------
//	GetMidiIn
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) LPMIDIEVENT WINAPI GetMidiInEx(HMIN hMidiIn)
#else
__declspec(dllexport) LPMIDIEVENT WINAPI GetMidiIn(HMIN hMidiIn)
#endif
{
	return GetMidiIn32(hMidiIn);
}

//----------------------------------------------------------------------------
//	OpenSync
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) HSYNC WINAPI OpenSyncEx(HSYNC hSync, HWND hWnd, WORD mode, WORD timerPeriod)
#else
__declspec(dllexport) HSYNC WINAPI OpenSync(HSYNC hSync, HWND hWnd, WORD mode, WORD timerPeriod)
#endif
{
	return OpenSync32(hSync, hWnd, mode, timerPeriod);
}

//----------------------------------------------------------------------------
//	CloseSync
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) WORD WINAPI CloseSyncEx(HSYNC hSync)
#else
__declspec(dllexport) WORD WINAPI CloseSync(HSYNC hSync)
#endif
{
	return CloseSync32(hSync);
}

//----------------------------------------------------------------------------
//	StopSync
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) void WINAPI StopSyncEx(HSYNC hSync)
#else
__declspec(dllexport) void WINAPI StopSync(HSYNC hSync)
#endif
{
	StopSync32(hSync);
}

//----------------------------------------------------------------------------
//	StartSync
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) void WINAPI StartSyncEx(HSYNC hSync)
#else
__declspec(dllexport) void WINAPI StartSync(HSYNC hSync)
#endif
{
	StartSync32(hSync);
}

//----------------------------------------------------------------------------
//	PauseSync
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) void WINAPI PauseSyncEx(HSYNC hSync, BOOL reset)
#else
__declspec(dllexport) void WINAPI PauseSync(HSYNC hSync, BOOL reset)
#endif
{
	PauseSync32(hSync, reset);
}

//----------------------------------------------------------------------------
//	ReStartSync
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) void WINAPI ReStartSyncEx(HSYNC hSync)
#else
__declspec(dllexport) void WINAPI ReStartSync(HSYNC hSync)
#endif
{
	ReStartSync32(hSync);
}

//----------------------------------------------------------------------------
//	SetTempo
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) WORD WINAPI SetTempoEx(HSYNC hSync, DWORD uSPerBeat)
#else
__declspec(dllexport) WORD WINAPI SetTempo(HSYNC hSync, DWORD uSPerBeat)
#endif
{
	return SetTempo32(hSync, uSPerBeat);
}

//----------------------------------------------------------------------------
//	SetResolution
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) void WINAPI SetResolutionEx(HSYNC hSync, WORD resolution)
#else
__declspec(dllexport) void WINAPI SetResolution(HSYNC hSync, WORD resolution)
#endif
{
	SetResolution32(hSync, resolution);
}

//----------------------------------------------------------------------------
//	SetBeatRatio
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) void WINAPI SetBeatRatioEx(HSYNC hSync, WORD beatrate, WORD qnoterate)
#else
__declspec(dllexport) void WINAPI SetBeatRatio(HSYNC hSync, WORD beatrate, WORD qnoterate)
#endif
{
	SetBeatRatio32(hSync, beatrate, qnoterate);
}

//----------------------------------------------------------------------------
//	GetTempo
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) DWORD WINAPI GetTempoEx(HSYNC hSync)
#else
__declspec(dllexport) DWORD WINAPI GetTempo(HSYNC hSync)
#endif
{
	return GetTempo32(hSync);
}

//----------------------------------------------------------------------------
//	GetResolution
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) WORD WINAPI GetResolutionEx(HSYNC hSync)
#else
__declspec(dllexport) WORD WINAPI GetResolution(HSYNC hSync)
#endif
{
	return GetResolution32(hSync);
}

//----------------------------------------------------------------------------
//	GetPosition
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) DWORD WINAPI GetPositionEx(HSYNC hSync, WORD units)
#else
__declspec(dllexport) DWORD WINAPI GetPosition(HSYNC hSync, WORD units)
#endif
{
	return GetPosition32(hSync, units);
}

// *** V1.55 ***
//----------------------------------------------------------------------------
//	GetMidiInFilters
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) DWORD WINAPI GetMidiInFiltersEx(HMIN hMidiIn)
#else
__declspec(dllexport) DWORD WINAPI GetMidiInFilters(HMIN hMidiIn)
#endif
{
	return GetMidiInFilters32(hMidiIn);
}

// *** V1.55 ***
//----------------------------------------------------------------------------
//	SetMidiInFilters
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) void WINAPI SetMidiInFiltersEx(HMIN hMidiIn, DWORD dwFilters)
#else
__declspec(dllexport) void WINAPI SetMidiInFilters(HMIN hMidiIn, DWORD dwFilters)
#endif
{
	SetMidiInFilters32(hMidiIn, dwFilters);
}

// *** V1.55 ***
//----------------------------------------------------------------------------
//	GetMidiOutFilters
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) DWORD WINAPI GetMidiOutFiltersEx(HMOUT hMidiOut)
#else
__declspec(dllexport) DWORD WINAPI GetMidiOutFilters(HMOUT hMidiOut)
#endif
{
	return GetMidiOutFilters32(hMidiOut);
}

// *** V1.55 ***
//----------------------------------------------------------------------------
//	SetMidiOutFilters
//----------------------------------------------------------------------------
#ifdef WIN95_DLL
__declspec(dllexport) void WINAPI SetMidiOutFiltersEx(HMOUT hMidiOut, DWORD dwFilters)
#else
__declspec(dllexport) void WINAPI SetMidiOutFilters(HMOUT hMidiOut, DWORD dwFilters)
#endif
{
	SetMidiOutFilters32(hMidiOut, dwFilters);
}
